//      Demo 5: SimpleBubSort.java

//            The Standard Simple Bubble Sort Demo
//    Reads in and Sorts an Array of Randomly Chosen Integers
//               A Java Screen Based Application

//  Based on MANY Similar Programs in a Wide Variety of Sources

//                by Charles W. Neville, 1982
//         Remarks Copyright Charles W. Neville, 1998

// Sorting an array means rearranging the elements of the array so they are in 
// order.  To be sortable, the elements of an array must be a (linearly)
// ORDERED DATA TYPE.  This simply means there is an order relation on the data 
// type satisfying the usual laws for the > relation:
//
//    (1) (transitivity) if a > b and b > c then a > c.
//    (2) (trichotomy) for a given pair a and b, exactly one of the following 
//        holds: a > b, a = b, or b > a.
//
// Bubble sort is one of many different sorting methods.  There is not much to 
// recommend bubble sort, except for its simplicity.  (In the early days of 
// computing when memory was very scarce, it also had the advantage that it 
// sorted "in place", that is it only required one array.)  Like the other 
// elementary sorts, bubble sort runs in O(n^2) time, both in the average and 
// worst cases.  However, it is probably the least efficient elementary sort, 
// taking perhaps twice as long to run as, for example, insertion sort.
//
// This program does not require any command line arguments

import java.util.*;  // for class Random

class SimpleBubSort {

   // make everything static so we don't have to create an object

   static void bubSort(int[] a) {

      int temp;

      // an array of length 1 is automatically sorted, 
      // so only sort arrays of length > 1
      if (a.length > 1) {
        // actual bubble sort routine
        for (int i = a.length - 1; i > 0; i--) {
           for (int j = 1; j <= i; j++) {
              if (a[j - 1] > a[j]) {
                 // swap a[j - 1] and a[j]
                 temp = a[j - 1];
                 a[j - 1] = a[j];
                 a[j] = temp;
              }
           }
         }
      }

   } // bubSort

   // print array a to screen, with spaces between integers
   // assumes array a contains positive integers of 3 digits or less
   static void nicePrint(int[] a) {
   
      for (int i = 0; i < a.length; i++) {
         if (a[i] < 10) {
            System.out.print("   ");  // 3 leading spaces
            System.out.print(a[i]);
         }
         else if (a[i] < 100) {
             System.out.print("  ");  // 2 leading spaces
             System.out.print(a[i]);
         }
         else {
              System.out.print(" ");  // 1 leading space
              System.out.print(a[i]);
         }
       }  // for

    }  // nicePrint


   // fill an array with 20 random integers in the range 0 through 99, sort it, 
   //and print it out to the screen.
   public static void main(String[] args) {

      int a[] = new int[20];  // array to be sorted

      Random random = new Random();  // random number generator object

      // fill array a with positive 3 digit random integers
      // (to simplify output formatting).
      for (int i = 0; i < a.length; i++) {
         a[i] = Math.abs(random.nextInt()) %  1000;
       }

       // document output
       System.out.println("Unsorted array is:");

       // echo print array a to screen
       nicePrint(a);

        // skip two lines
        System.out.println();
        System.out.println();

        // sort array a
        bubSort(a);

        System.out.println("Sorted array is:");

        // print sorted array to screen
        nicePrint(a);

     } // main

} // class SimpleBubSort

      
